//
//  DemoMountianchart.swift
//  TRAQ
//
//  Created by Development Kaha on 02/07/19.
//  Copyright © 2019 Kaha Pte .Ltd. All rights reserved.
//

import Foundation
import UIKit
import SciChart

final class MountianChartExample: UIViewController {

    private var sciChartSurface: SCIChartSurface!
    private let labelProvider = xAxisLabelProviderExample()
    let MaxVisibleCount = 7
    private var sampleVaues: [Double] {
        return (0...6).map {_ in .random(in: 60.0...150.0)}
    }

    override func viewDidLoad() {
        super.viewDidLoad()
        labelProvider.dates = sampleVaues
        addChartSurface()
        setData()
    }

    fileprivate  func setGraphMaxVisibleRange(_ xAxis: SCINumericAxis) {
        let count = sampleVaues.count
        let visibleMaxRange: Double = count > MaxVisibleCount ? Double(MaxVisibleCount) - 0.5 : Double(count) - 0.5
        xAxis.visibleRange = SCIDoubleRange(min: SCIGeneric(-0.5), max: SCIGeneric(visibleMaxRange))
    }

    private func addChartSurface() {
        sciChartSurface = SCIChartSurface(frame: .zero)
        sciChartSurface.translatesAutoresizingMaskIntoConstraints = false
        sciChartSurface.isOpaque = false

        let fontStyle = SCITextFormattingStyle()
        fontStyle.color = UIColor.black

        let xAxis = SCINumericAxis()
        xAxis.visibleRange = SCIDoubleRange(min: SCIGeneric(-0.5), max: SCIGeneric(6.5))
        xAxis.style.drawMajorGridLines = false
        xAxis.style.drawMinorGridLines = false
        xAxis.style.drawMajorBands = false
        xAxis.style.drawMinorTicks = false
        xAxis.labelProvider = labelProvider
        xAxis.style.labelStyle = fontStyle

        setGraphMaxVisibleRange(xAxis)
        sciChartSurface.xAxes.add(xAxis)
        let yAxis = SCINumericAxis()
        yAxis.axisAlignment = .left
        xAxis.style.drawMajorTicks = false
        yAxis.style.drawMajorGridLines = false
        yAxis.style.drawMinorGridLines = false
        yAxis.style.drawMajorBands = false
        yAxis.style.drawMinorTicks = false
        yAxis.style.drawMajorTicks = false
        yAxis.style.labelStyle = fontStyle
        sciChartSurface.yAxes.add(yAxis)

        sciChartSurface.renderableSeriesAreaFill = SCISolidBrushStyle(color: .white)
        sciChartSurface.backgroundColor = .white

        let allViews: [String: Any] = [
            "view": self.view,
            "sciChartSurface": sciChartSurface
        ]
        self.view.addSubview(sciChartSurface)
        
        let rollOverModifier = SCIRolloverModifier()
        rollOverModifier.style.contentPadding = 0
        rollOverModifier.style.tooltipBorderWidth = 0
        rollOverModifier.style.rolloverPen = .none
        rollOverModifier.style.showAxisLabels = false

        let xAxisDragModifier = SCIXAxisDragModifier()
        xAxisDragModifier.clipModeX = .clipAtExtents
        xAxisDragModifier.dragMode = .pan

        let groupModifiers = SCIChartModifierCollection(childModifiers: [rollOverModifier, xAxisDragModifier])
        sciChartSurface.chartModifiers = groupModifiers
        let horizontalConstraint = NSLayoutConstraint.constraints(withVisualFormat: "H:|[sciChartSurface]|", options: [], metrics: nil, views: allViews)
        let verticalConstraint = NSLayoutConstraint.constraints(withVisualFormat: "V:|[sciChartSurface]|", options: [], metrics: nil, views: allViews)
        NSLayoutConstraint.activate(horizontalConstraint + verticalConstraint)
    }

    private func setData() {

        //Create Rest Data set avg data set a nd max HR data set

        let restDataSet = SCIXyDataSeries(xType: .double, yType: .double)
        restDataSet.seriesName = "Rest Data"
        let avgDataSet = SCIXyDataSeries(xType: .double, yType: .double)
        avgDataSet.seriesName = "Average Data"
        let maxDataSet = SCIXyDataSeries(xType: .double, yType: .double)
        maxDataSet.seriesName = "Max Data"

        for i in 0..<sampleVaues.count {
            let restHR: Double = 0
            restDataSet.appendX(SCIGeneric(i), y: SCIGeneric(restHR))
            let averageHR = sampleVaues[i] * 0.5
            avgDataSet.appendX(SCIGeneric(i), y: SCIGeneric(averageHR))

            maxDataSet.appendX(SCIGeneric(i), y: SCIGeneric(sampleVaues[i] * 0.8))
        }

        let restDataSeries = RestSeriesEx()
        let avgDataSeries = AvgSeriesEx()
        let maxDataSeries = MaxSeriesEx()
       

        restDataSeries.dataSeries = restDataSet
        restDataSeries.strokeStyle = SCISolidPenStyle(color: .green, withThickness: 1)
        restDataSeries.areaStyle.color = UIColor.clear
        restDataSeries.addAnimation(SCIWaveRenderableSeriesAnimation(duration: 1.0, curveAnimation: .easeOut))

        avgDataSeries.dataSeries = avgDataSet
        avgDataSeries.strokeStyle = SCISolidPenStyle(color: .blue, withThickness: 1)
        avgDataSeries.areaStyle.color = UIColor.clear
        avgDataSeries.addAnimation(SCIWaveRenderableSeriesAnimation(duration: 1.0, curveAnimation: .easeOut))


        maxDataSeries.dataSeries = maxDataSet
        maxDataSeries.strokeStyle = SCISolidPenStyle(color: .red, withThickness: 1)
        maxDataSeries.areaStyle = SCILinearGradientBrushStyle(colorStart: .red, finish: .white, direction: .vertical)
        maxDataSeries.addAnimation(SCIWaveRenderableSeriesAnimation(duration: 1.0, curveAnimation: .easeOut))


        sciChartSurface.renderableSeries.add(maxDataSeries)
        sciChartSurface.renderableSeries.add(restDataSeries)
        sciChartSurface.renderableSeries.add(avgDataSeries)

    }
}

class xAxisLabelProviderExample: SCINumericLabelProvider {
    private let weeks = ["1W", "2W", "3W", "4W", "5W","6W"]
    var dates: [Double] = []

    override func formatLabel(_ dataValue: SCIGenericType) -> NSAttributedString! {
        let doubleData = SCIGenericDouble(dataValue)
        debugPrint("Data in xAxis Label provider is \(doubleData)")
        if floor(doubleData) == doubleData {
            debugPrint("Data is an integer value")
            let data = Int(doubleData)
            var index = 0
            debugPrint("Is dates empty = \(dates.isEmpty)")
            guard !dates.isEmpty else { return NSAttributedString(string: "\(data)") }
            index = data % dates.count
            debugPrint("Index is \(index)")
            let timeString = "\(dates[index])"
            return NSAttributedString(string: timeString)
        } else {
            debugPrint("Not an integer....returning empty string")
            return NSAttributedString(string: "")
        }
    }
}

//MARK:
fileprivate class MaxSeriesInfoEX: SCIXySeriesInfo {
    var toolTipDisplayString: ((String) -> (String, String))!
    
    convenience init(series: SCIRenderableSeriesProtocol!, hitTest: SCIHitTestInfo, displayBlock: @escaping (String) -> (String, String)) {
        self.init(series: series, hitTest: hitTest)
        self.toolTipDisplayString = displayBlock
    }
    
    override func createDataSeriesView() -> SCITooltipDataView! {
        let view : ColumnChartCustomSeriesInfo = ColumnChartCustomSeriesInfo.createInstance() as! ColumnChartCustomSeriesInfo
        let (xVal, yVal) = toolTipDisplayString(self.formatXCursorValue(self.xValue()))
        view.topLabel.text = yVal
        view.bottomLabel.text = xVal
        return view
    }
}

fileprivate class MaxSeriesEx: SCIFastMountainRenderableSeries {
    var toolTipDisplayString: ((String) -> (String, String))!
    
//    override func toSeriesInfo(withHitTest info: SCIHitTestInfo) -> SCISeriesInfo! {
//        return MaxSeriesInfoEX(series: self, hitTest: info, displayBlock: toolTipDisplayString)
//    }
}

fileprivate class AvgSeriesEx: SCIFastMountainRenderableSeries {
    var toolTipDisplayString: ((String) -> (String, String))!
    
//    override func toSeriesInfo(withHitTest info: SCIHitTestInfo) -> SCISeriesInfo! {
//        return MaxSeriesInfoEX(series: self, hitTest: info, displayBlock: toolTipDisplayString)
//    }
}

fileprivate class RestSeriesEx: SCIFastMountainRenderableSeries {
    var toolTipDisplayString: ((String) -> (String, String))!
    
//    override func toSeriesInfo(withHitTest info: SCIHitTestInfo) -> SCISeriesInfo! {
//        return MaxSeriesInfoEX(series: self, hitTest: info, displayBlock: toolTipDisplayString)
//    }
}
